function data = MSR780_GetSpectrum(channel)
% function data = MSR780_GetSpectrum(channel)
% Gets data from analyzer channel. channel is 0 or 1 for channel A or B.
% data is a structure data.{data,freq,units}
%
% To replicate the display use plot(data.freq,data.data).
% channel is 0 (A) or 1 (B).
% data.units are the units as a character string.
%
% More about units. You can use V^2 units, dBVrms, dBm, etc...
% These dB units are very confusing.
% dBVrms is 20*log(rms(V/1V)), so a signal with Vrms=1V (a sinusoid with
% amplitude of 1.41V) is a signal with 0 dBVrms at its frequency.
% the same sinusoid with 1/10 the amplitude and 1/100 the power is
% -20dBVrms, so the dB refers to the power of the signal in units of power, 
% 10dB is a factor of 10 in power 
%
% if the analyzer communication becomes unsynchronized, use MSR780_Reset.
%
% to make the SR780 talk to GPIB, you must set the output to GPIB,
% otherwise it will try to send GPIB answers over the RS232 serial port.
% This setting is reset on power off!
% SR780 uses EOI end of message marker

DEVICE_NAME=MSR780_DefaultName;

if (nargin<1) , 
    channel=0;
    %error('Wrong number of arguments\n'); 
end
if length(channel)>1, error('only supports single channel aquisition'); end;

cmd=sprintf('DSPN?%d',channel);
GPIB_Write(cmd,DEVICE_NAME);
returnString=GPIB_Read(DEVICE_NAME);
nPoints=eval(returnString);

cmd=sprintf('DSPY?%d',channel);
GPIB_Write(cmd,DEVICE_NAME);
returnString=GPIB_Read(DEVICE_NAME);
ydata=sscanf(returnString,'%g,',inf);  
ydata=ydata'; % make data a row vector
%eval(['[',returnString(1:end-1),']']);  % values are comma separated, discard EOL

% get measurement group in order to determine xaxis values

cmd=sprintf('MGRP?%d',channel);
GPIB_Write(cmd,DEVICE_NAME);
returnString=GPIB_Read(DEVICE_NAME);
measurementGroup=eval(returnString);

switch measurementGroup,
case 0,
    % comes here for FFT
    
    cmd=sprintf('FSTR?%d',channel);
    GPIB_Write(cmd,DEVICE_NAME);
    returnString=GPIB_Read(DEVICE_NAME);
    startFreq=eval(returnString);
    sscanf(returnString,'%g',inf);  
    
    
    cmd=sprintf('FEND?%d',channel);
    GPIB_Write(cmd,DEVICE_NAME);
    returnString=GPIB_Read(DEVICE_NAME);
    endFreq=eval(returnString);  
    
    xdata=linspace(startFreq,endFreq,length(ydata));
case 2,
    % comes here for swept sine measurement
    
    cmd=sprintf('SSTR?%d',channel);
    GPIB_Write(cmd,DEVICE_NAME);
    returnString=GPIB_Read(DEVICE_NAME);
    startFreq=eval(returnString);
        
    
    cmd=sprintf('SSTP?%d',channel);
    GPIB_Write(cmd,DEVICE_NAME);
    returnString=GPIB_Read(DEVICE_NAME);
    endFreq=eval(returnString);  
    
    cmd=sprintf('SSTY?%d',channel);
    GPIB_Write(cmd,DEVICE_NAME);
    returnString=GPIB_Read(DEVICE_NAME);
    sweepStyle=eval(returnString);  
    
    if sweepStyle==0,
        xdata=linspace(startFreq,endFreq,length(ydata));
    else,
        xdata=logspace(log10(startFreq),log10(endFreq),length(ydata));
    end
    
otherwise,
    error('unsupported measurement group');
end


cmd=sprintf('UNIT?%d',channel);
GPIB_Write(cmd,DEVICE_NAME);
returnString=GPIB_Read(DEVICE_NAME);
unitNumber=eval(returnString);
switch unitNumber,
    case 0, unit='Vpk';
    case 1, unit='Vrms';
    case 2, unit='Vpk^2';
    case 3, unit='Vrms^2';
    case 4, unit='dBVpk';
    case 5, unit='dBVrms';
    case 6, unit='dBm';
    case 8, unit='degrees'; % phase plot
    case 9, unit='radians';
    case 10, unit='Units'; % xfer function
    case 11, unit='dB';  
    otherwise, fprintf('unknown units for data'); 
end

data=struct('data',ydata,'freq',xdata,'units',unit);







