function [result,sdev,unsettled]=Take182K617_DVM(K617Name)
% function [result,sdev,unsettled]=K617_DVM([K617Name])
% Measures using the K617 ammeter using the current
% settings, returns measurement as result, along with standard
% deviation of measurments and a flag telling whether
% settling was achieved. The user is 
% responsible for the K617 being in the right 
% mode.
%
%	The variables SettleTol, NumSettle, and NumAvg
% determine the required accuracy.
%  SettleTol is the settling tolerance, criterion:
%   abs(val - lastval) < SettleTol*abs(val)
% 	NumSettle is the number of times K617_DVM 
%		tries to get the results to settle before giving up.
% 		settling means that the results change direction.
%	Numavg is the number of results that are averaged
% 		for the final result.
% if you want to change these values, declare them global
% with 'global NumSettle Numavg' and set them
% to the desired values.
%
% if NumAvg==0 | NumAvg ==1, then K617_DVM_quick is called.
% if NumSettle == 0 then no settling is performed and
% unsettled is always 0.
% if NumAvg == 1 then sdev is always 0.
% if K617AutoRange ==1 then 617 is used in autoranging mode
% (much faster).  if K617AutoRange==0, 
%
% A global variable called K617range can be used to keep
% track of previous range value to speed up finetaking
% of data. If this variable does not exist a new range
% decision (from autoranging) is made for each data point.
%
% Also see : K617_SetMode, K617_DVM_quick

% initialize params

global SettleTol NumAvg NumSettle Verbose 
global WaitTime InitWaitTime
global K617AutoRange


if isempty(SettleTol), SettleTol=0.01; end;
if isempty(NumSettle), NumSettle=5; end;
if (NumSettle > 0),
  if isempty(SettleTol), SettleTol=inf; end;
end;
if isempty(NumAvg), NumAvg=5; end;
if isempty(Verbose), Verbose=0; end;
if nargin==0, K617Name=K617_DefaultName; end
if isempty(WaitTime), WaitTime = 0.36; end;
if isempty(InitWaitTime), InitWaitTime = 5; end;
if isempty(K617AutoRange), K617AutoRange = 1; end;


unsettled = 0;

if K617AutoRange,
	GPIB_Write('R0X',K617Name); % turn on autoranging
else,
	% set range without K617 autoranging
	[oldvalue,K617range,overflow]=K617AutoSetRange(K617Name);
end;

if NumAvg==0 | NumAvg==1, % do a quick and dirty take
	result=K617_DVM_quick(K617Name);
	sdev=0;
	return;
end

%-------------------------------------------------------------------------
% get the  initial change
if NumSettle>=2,
	if Verbose, Take182('msg','Settling'); end;
	oldvalue=K617_DVM_quick(K617Name);
	value=K617_dvm_quick(K617Name);
	diff = abs( value - oldvalue);
	count=0;
	value=oldvalue;
	
	while diff>SettleTol*abs(value),
		if(count>NumSettle),
			sysbeep;
			s=sprintf('K617 did not settle to %2.1f%% within %d readings',SettleTol*100, NumSettle);
			Take182('warn',s);
			unsettled = 1;
			break;
		end
		unsettled = 0;	
		count=count+1;
		wait(WaitTime);
		oldvalue=value;
		value=K617_DVM_quick(K617Name);
		diff = abs(oldvalue - value);
	end % whileloop settling
end; % settling

%------------------------------------------------------------------------------
% now average - use last two settle vals too if settling on
% dialog guarentees that NumSettle will be either zero or 3 or bigger
% also NumAvg will be at least 3 if NumSettle >=3

res=zeros(1,NumAvg);
if Verbose, Take182('msg',' Averaging'); end
if NumSettle==0,
	start=1;
else, % assume NumSettle is >=3
	res(1)=oldvalue; res(2)=value;
	start=3;
end;

for i=start:NumAvg,
	[res(i),overflow]=K617_DVM_quick(K617Name);
end

result=mean(res(1:NumAvg));
sdev=std(res);

return



%	AllRanges = [200e-3  2e-12;     
%		2       20e-12;   
%		20      200e-12;  
%		200     2e-9;
%		200     20e-9;
%		200     200e-9;
%		200     2e-6;
%		200     20e-6;
%		200     200e-6;
%		200     2e-3;
%		200     20e-3];



