function [a,b,chisq,A,lam]=GUIfit182Mrqmin(x,y,sig,A,indx,func,relax,a,b,lam,ochisq)
%Levenberg-Marquardt method to reduce chisqr of a non-linear fit
%to data.  Based on Numerical Recipes page 526. This function calls
% mrqcof (which calls func).
%
% Usage: [a,b,chisq,B,lam]=GUIfit182Mrqmin(x,y,sig,A,indx,func,a,b,lam,relax)
%          where: x & y are independent and dependent data
%                 sig are uncertainties in y
%                 A are initial parameters
%                 indx are indexes for parameters to adjust
%                 func is function to fit
%                 a & b are the covariance matrix and solution vector
%                 lam is L-M fitting parameter
%                 relax is relaxation coefficient
%                 B is an improved set of coefficients
%  first time call this routine with 7 arguments
%  This routine is repeatedly called until convergence is achieved
%

% J.M. Brown brown@geophys.washington.edu
%            Geophysics Program AK-50, University of Washington,
%            Seattle 98195 (206) 543-9419
% 5/5/90

if (nargin~=7 & nargin~=11),
    error('GUIfit182Mrqmin requires seven or eleven arguments'), end

% Make sure all vectors are column vectors
 x=x(:);
 y=y(:);
 sig=sig(:);
 atry=A(:);

% first time through initialize variables
  if nargin==7,
     lam=.001;
     A=A(:);
     atry=A;
     [a,b,chisq]=GUIfit182Mrqcof(x,y,sig,A,indx,func);
     ochisq=chisq;
   end


% calculate marquardt matrix:
  if length(indx)==1,
     a_lam=(1+lam)*a;
  else,
     a_lam=(lam*eye(a)+ones(size(a))).*a;
  end

% solve for increments of parameters:
  da=a_lam\b;
  atry(indx)=A(indx)+relax*da;

% try new parameters:
  [a_out,da,chisq]=GUIfit182Mrqcof(x,y,sig,atry,indx,func);

% check to see if chisq is reduced (if so reduce lam):
  if chisq<ochisq,
    lam=.1*lam;
    a=a_out;
    b=da;
    A(indx)=atry(indx);

% if chisq is not smaller increase lam and return:
  else
    lam=10*lam;
    chisq=ochisq;
  end
