/*
 *...........................ANALOG INPUT FUNCTIONS...........................
 *
 */

#include "NI_DAQ_MACStructs.h"
#include "OpenBoard.h"

/*
 * AI_Config -- configure analog channel
 */	
NBErrType AI_Config(int16 slot, int16 mode, int16 range, int16 polar)
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	NBErrType err;
 
 	/* conditionally open slot */
 	if ((err = OpenBoard (slot)) == noErr) {
		/*
		 * make AIConfig Control call:
		 *		- parameters of: slot, mode, range, polarity
		 */
		apbPtr = &apb;
		apb.slot = slot;	
		apb.mode = (mode ? modeBit : 0) | (polar ? polarity : 0);
		apb.range = range;
		
		LDSysError = Control(ref_table[slot], AIConfigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* AI_Config */

/*
 * AI_Read -- make single analog reading, return in 'value'
 */
NBErrType AI_Read(int16 slot, int16 chan, int16 gain, int16 *value)
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	NBErrType err;
 
 	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 * make AIRead Control call:
		 *		- parameters of: slot, chan, gain, and value(output)
		 */
		apbPtr = &apb;
		apb.slot = slot;	
		apb.chan = chan;
		apb.gain = gain;
		LDSysError = Control(ref_table[slot], AIReadID, (Ptr)&apbPtr);
		
		ProcError(LDSysError, err, apb.error);
	
		*value = apb.value;	/* analog input reading */
	}
	return err;
} /* AI_Read */


/*
 * AI_Scale -- scale 12-bit binary reading to 32-bit float voltage
 */	
NBErrType AI_Scale(int16 slot, int16 gain, int16 reading, float *voltage)
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */	
	NBErrType err;
 
 	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 * make AIScale Control call:
		 *		- parameters : slot, gain, reading(input), voltage(output)
		 */
		apbPtr = &apb;
		apb.slot = slot;	
		apb.gain = gain;
		apb.value = reading;
		LDSysError = Control(ref_table[slot], AIScaleID, (Ptr)&apbPtr);
		
		ProcError(LDSysError, err, apb.error);
	
		*voltage = apb.voltage;	/* analog input reading */
	}
	return err;
} /* AI_Scale */

/*
 * AI_Setup -- 	selects an analog input channel and gain setting 
 *				for later conversion 
 */
NBErrType AI_Setup(int16 slot, int16 chan, int16 gain)
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */	
	NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make AISetup Control call:
		 *		- parameters : slot, channel, gain
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.gain = gain;
		apb.chan = chan;
		LDSysError = Control(ref_table[slot], AISetupID, (Ptr)&apbPtr);	
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* AI_Setup */

/*
 * AI_Check -- 	checks status of the board to see if an A/D conversion is 
 *				available to be read
 */
NBErrType AI_Check(int16 slot, int16 *status, int16 *reading)
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */	
	NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make AI_Check Control call:
		 *		- parameters : slot, status(output), reading(output)
		 */
		apbPtr = &apb;
		apb.slot = slot;
		LDSysError = Control(ref_table[slot], AICheckID, (Ptr)&apbPtr); /*xxx*/
	
		ProcError(LDSysError, err, apb.error);
	
		/* return values */
		if(*status = apb.status) 
			*reading = apb.value;
	}
	return err;
} /* AI_Check */

/*
 * AI_Clear -- clears analog input circuitry
 */
NBErrType AI_Clear(int16 slot)
	{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */	
	NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make AIClear Control call:
		 *		- parameters : slot
		 */
		apbPtr = &apb;
		apb.slot = slot;
		LDSysError = Control(ref_table[slot], AIClearID, (Ptr)&apbPtr); /*xxx*/
		ProcError(LDSysError, err, apb.error);
	}
	return err;
	} /* AI_Clear */
	
/*
 * AI_Mux_Config -- configure analog multiplexed input using AMUX-64('s). Call this before
 *					DAQ (and SCAN) routines and range-checking and acquisition will use
 *					AMUX support.
 */	
 
NBErrType AI_Mux_Config (int16 slot, int16 muxNum)

	/*  muxNum is the # of AMUX boards present (0,1,2 or 4) */
{
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	NBErrType err;
 
 	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {	 
		apbPtr = &apb;
		apb.slot = slot;	
		apb.value = muxNum;
		
		LDSysError = Control(ref_table[slot], AIMuxConfigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
	
} /* AI_Mux_Config */