/*
 *...........................DATA ACQUISITION FUNCTIONS...........................
 */

#include "NI_DAQ_MACStructs.h"
#include "OpenBoard.h"

/* 
 *	DAQ_Config --	stores configuration information for subsequent data
 *					acquisition operations
 */
NBErrType DAQ_Config(int16 slot, int16 ext_trig, int16 ext_gate, int16 ext_conv)
{	
	AnalogParamBlock apb, *apbPtr;		/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*	
		 *	make DAQ_Config Control call:
		 *		- parameters : slot, ext_trig, ext_gate, ext_conv
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.mode = (ext_trig ? exttrig : 0) | (ext_gate ? extgate : 0) |
				   (ext_conv ? extconv : 0);
		LDSysError = Control(ref_table[slot], DAQConfigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_Config */

/* 
 *	DAQ_Start --	initiates a single channel data acquisition operation
 */
NBErrType DAQ_Start(int16 slot, int16 chan, int16 gain,int16 *buffer, int32 Count, int16 timebase, int32 samp_interval)

{	
	AnalogParamBlock apb, *apbPtr;			/* allocate Analog parameter block */
	register NBErrType err;
		
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make DAQ_Start Control call:
		 *		- parameters : (slot, chan, gain, buffer, Count, timebase, samp_interval)
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = chan;
		apb.gain = gain;
		apb.buffer = (LD_Ptr) buffer;
		apb.cnt = Count;
		apb.timebase = timebase;
		apb.samp_int = samp_interval;
		
		LDSysError = Control(ref_table[slot], DAQStartID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_Start */

/* 
 *	DAQ_Check --	checks to see whether the current data acquisition operation is 
 *					complete and returns status and array of acquired data if
 *					it is complete
 */
NBErrType DAQ_Check(int16 slot, int16 *status)
{	
	AnalogParamBlock apb, *apbPtr;			/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make DAQCheck Control call:
		 *		- parameters : slot, status
		 */
		
		apbPtr = &apb;
		apb.slot = slot;

		LDSysError = Control(ref_table[slot], DAQCheckID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
		/*return values*/
		*status = apb.status;
	}
	return err;
} /* DAQ_Check */

/* 
 *	DAQ_Clear --	cancels current data acquisition operations and
 *					reinitializes the data acquistion circuitry
 */
NBErrType DAQ_Clear(int16 slot)
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;
		
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make DAQ_Clear Control call:
		 *		- parameters : slot
		 */
		apbPtr = &apb;
		apb.slot = slot;
		
		LDSysError = Control(ref_table[slot], DAQClearID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_Clear */

/* 
 *	DAQ_Scale --	given an array of acquired 12-bit data and gain for that
 *					data, converts array to actual voltages measured
 */
NBErrType DAQ_Scale(int16 slot, int16 gain, int32 Count, int16 *bin_array, float *volt_array)
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make DAQ_Scale Control call:
		 *		- parameters : slot, gain, Count, bin_array, volt_array
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.gain = gain;
		apb.cnt = Count;
		apb.buffer = (LD_Ptr) bin_array;
		apb.data = (LD_Ptr) volt_array;
		
		LDSysError = Control(ref_table[slot], DAQScaleID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_Scale */

/* 
 *	SCAN_Setup --	initializes circuitry on board for a scanned data
 *					acquisition operation.  Initialization includes storing
 *					a table of the channel sequence and gain setting for
 *					each channel on the board.
 */
NBErrType SCAN_Setup(int16 slot, int16 num_chans, int16 *chan_vector, int16 *gain_vector, int16 mux_mode)
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make SCAN_Setup Control call:
		 *		- parameters : slot, num_chans, chan_vector, gain_vector, mux_mode
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = num_chans;
		apb.buffer = (LD_Ptr) chan_vector;
		apb.data = (LD_Ptr) gain_vector;
		apb.value = mux_mode;
		
		LDSysError = Control(ref_table[slot], SCANSetupID, (Ptr)&apbPtr);	
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* SCAN_Setup */

/* 
 *	SCAN_Start --	initiates a multiple channel data acquisition operation
 */
NBErrType SCAN_Start(int16 slot,int16  *buffer, int32 Count, int16 timebase, int32 samp_interval)
/*int16 slot, *buffer;
int32 Count;
int16 timebase;
int32 samp_interval;*/
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;

	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.buffer = (LD_Ptr) buffer;
		apb.cnt = Count;
		apb.timebase = timebase;
		apb.samp_int = samp_interval;
		
		LDSysError = Control(ref_table[slot], SCANStartID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* SCAN_Start */

/* 
 *	SCAN_Check --	checks to see whether the current scanned data acquisition
 *					operation is complete and returns status
 */
NBErrType SCAN_Check(int16 slot, int16 *status)
/*int16 slot, *status;*/
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		
		LDSysError = Control(ref_table[slot], SCANCheckID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	
		/* return values */
		*status = apb.status;
	}
	return err;
} /* SCAN_Check */

/* 
 *	SCAN_Demux --	demultiplexes the buffer and copies data to two-dimensional array
 */
NBErrType SCAN_Demux(int16 slot, int16 *buffer, int32 Count, int16 *parray[])
/*int16 slot, *buffer;
int32 Count;
int16 *parray[];*/
{	
	AnalogParamBlock apb, *apbPtr;	/* allocate Analog parameter block */
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.buffer = (LD_Ptr) buffer;
		apb.cnt = Count;
		apb.data = (LD_Ptr) parray;
		
		LDSysError = Control(ref_table[slot], SCANDemuxID, (Ptr)&apbPtr);	
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* SCAN_Demux */

/*                                   *** LAB-NB *** 
 *	Lab_SCAN_Start --	initiates a single channel data acquisition operation on Lab-NB
 */


NBErrType Lab_SCAN_Start(int16 slot, int16 num_chans, int16 gain, int16 *buffer, int32 count_per_chan, int16 timebase, int32 samp_interval)
/*int16 slot, num_chans, gain, *buffer;
int32 count_per_chan;
int16 timebase;
int32 samp_interval;*/
{	
	AnalogParamBlock apb, *apbPtr;			/* allocate Analog parameter block */
	register NBErrType err;
		
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		/*
		 *	make Lab_SCAN_Start Control call:
		 *		- parameters : (slot, num_chans, gain, buffer, count_per_chan, timebase, samp_interval)
		 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = num_chans;
		apb.gain = gain;
		apb.buffer = (LD_Ptr) buffer;
		apb.cnt = count_per_chan;
		apb.timebase = timebase;
		apb.samp_int = samp_interval;
		
		LDSysError = Control(ref_table[slot], LabSCANStartID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
	} /* Lab_SCAN_Start */


/*       
 * Lab_SCAN_Check -- returns information regarding a multichannel data acquisition 
 *                 
 */	
NBErrType Lab_SCAN_Check(int16 slot, int16 *status, int32 *retrieved,  int16 *SCAN_order)
{
	AnalogParamBlock apb, *apbPtr;			/* allocate Analog parameter block */
	register NBErrType err;
		
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
	
	/* 
	 *  make Lab_SCAN_Check control call
	 *       - parameters : (slot ,status, retrieved, SCAN_order)
	 */
		apbPtr = &apb;
		apb.slot = slot;
		apb.buffer = (LD_Ptr)SCAN_order;
		
		LDSysError = Control(ref_table[slot], LabSCANCheckID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
		*status = apb.status ;
		*retrieved = apb.cnt;
	}
	return err;

}  /* Lab_SCAN_Check */	
/*
 *	DAQ2Config - configures for double buffered acquisition and stores configuration
 *				 information for subsequent data acquisition operations.
 */

NBErrType DAQ2Config (int16 slot,int16 mode, int32 bufSize, int32 blkCount)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.mode = mode;
		apb.bufSize = bufSize;
		apb.blkCount = blkCount;
		
		LDSysError = Control(ref_table[slot], DAQ2ConfigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2Config */

/*
 *	DAQ2Get - reads blkCount bytes from the acquisition buffer and copies the block into
 * 				  buffer. If blkCount bytes are not available, the routine blocks, waiting for
 *				  the bytes. The acquisition buffer's tail pointer is updated.If triggering is 
 *				  enabled, DAQ2Get scans the acquisition buffer
 *				  from the tail pointer on and returns a block beginning where the 
 *				  trigger value occurred.
 */


NBErrType DAQ2Get (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 blkCount, int32 timeout)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = trig_chan;
		apb.mode = slope;
		apb.value = value;
		apb.buffer = (LD_Ptr) buffer;
		apb.blkCount = blkCount;
		apb.cnt = timeout;
		apb.samp_int = 0;	/* added v2.31 : return block beginning with trigger */
		
		LDSysError = Control(ref_table[slot], DAQ2GetID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2Get */

/*
 *	DAQ2Tap - reads the last blkCount bytes acquired and copies the block into buffer.
 * 				  If blkCount bytes are not available, the routine returns an error. The 
 *				  acquisition buffer's tail pointer is not updated. If triggering is enabled,
 *				  DAQ2Tap scans the acquisition buffer from the head pointer 
 *				  back and returns the latest block in which the trigger value occurred.
 */				  
 				

NBErrType DAQ2Tap (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 blkCount)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = trig_chan;
		apb.mode = slope;
		apb.value = value;
		apb.buffer = (LD_Ptr) buffer;
		apb.blkCount = blkCount;
		apb.samp_int = 0;	/* added v2.31 : return block beginning with trigger */
		
		LDSysError = Control(ref_table[slot], DAQ2TapID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2Tap */

/*
 *	DAQ2TGet	  -	reads blkCount samples from the acquisition buffer and copies
 *					the block into buffer.  If blkCount samples are not available,
 *					the routine blocks, waiting for the bytes.  The acquisition
 *					buffer's tail pointer is updated.  Repeated calls to DAQGetBlock
 *					return sequential blocks of data from the acquisition buffer.
 *					If triggering is enabled, DAQGetBlock scans the acquisition buffer
 *					from "trigger position" samples after the tail pointer on and returns 
 *					the block in which the trigger occurred, with the trigger at the 
 *					desired position. If there are not enough samples left to fill the 
 *					buffer, the actual number acquired is returned.
 *
 */

NBErrType DAQ2TGet (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 trigPos, int32 blkCount, int32 *actualCnt, int32 timeout)
/*int16 slot, trig_chan, slope, value, *buffer;
int32 blkCount, timeout, trigPos, *actualCnt;*/
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = trig_chan;
		apb.mode = slope;
		apb.value = value;
		apb.buffer = (LD_Ptr) buffer;
		apb.samp_int = trigPos;
		apb.blkCount = blkCount;
		apb.cnt = timeout;
			
		LDSysError = Control(ref_table[slot], DAQ2TGetID, (Ptr)&apbPtr);
		
		ProcError(LDSysError, err, apb.error);
		*actualCnt = apb.blkCount;
	}
	return err;
} /* DAQ2TGet */

/*
 *	DAQ2TTap -	 returns the most recent block of data acquired from the
 *				 background double-buffered data acquisition initiated by 
 *				 DAQ2Start.  Allows trigger to be defined anywhere within
 *				 the block of data.
 */	

NBErrType DAQ2TTap (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 trigPos, int32 blkCount)
/*int16 slot, trig_chan, slope, value, *buffer;
int32 trigPos, blkCount;*/
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = trig_chan;
		apb.mode = slope;
		apb.value = value;
		apb.buffer = (LD_Ptr) buffer;
		apb.blkCount = blkCount;
		apb.samp_int = trigPos;
		
		LDSysError = Control(ref_table[slot], DAQ2TTapID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2TTap */


/*
 *	DAQ2Clear - cancels and clears any current data acquisition, 
 *				  reinitializes the data acquisition circuitry,
 *				  deallocates buffer allocated by DAQ2Config and
 *				  disables double buffered data acquisition.
 */
 
NBErrType DAQ2Clear (int16 slot)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		
		LDSysError = Control(ref_table[slot], DAQ2ClearID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2Clear */
 

/*
 *	DAQ_Trigger  - enables or disables analog trigger on subsequent data acquisition
 *				   operations.
 */
 
NBErrType DAQ_Trigger (int16 slot, int16 trig_chan, int16 slope, int16 value, int32 timeout)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.chan = trig_chan;
		apb.mode = slope;
		apb.value = value;
		apb.cnt = timeout;
		
		LDSysError = Control(ref_table[slot], DAQTriggerID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_Trigger */


/*
 *	DAQ2MemConfig - 	configure double buffered data acquisitions to use a 
 *					Nubus memory expansion card for the acquisition buffer.
 */
 
NBErrType DAQ2MemConfig (int16 slot, int16 value,int16 mode)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.value = value;
		apb.mode = mode;
		
		LDSysError = Control(ref_table[slot], DAQ2MemConfigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ2MemConfig */

/*
 *	DAQ_PreTrig  - setup analog input pre-trigger information for stopping
 *				   single-buffered data acquisition when a specified number
 *				   of samples have been acquired after the occurrence of an
 *				   external stop trigger. (NB-MIO-16X only)
 */
 
NBErrType DAQ_PreTrig (int16 slot, int16 altTrig, int32 cnt)
{
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.value = altTrig;
		apb.cnt = cnt;
		
		LDSysError = Control(ref_table[slot], DAQPreTrigID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* DAQ_PreTrig */

/*
 *	SCAN_IntStart  - Initiates a multiple channel scanned data acquisition
 *					 and allows a sampling interval between channels and
 *					 a scanning interval between each sequence of channels
 *					 to be defined.  (NB-MIO-16X only)
 */
 
NBErrType SCAN_IntStart (int16 slot, int16 *buffer, int32 cnt, int16 sampTime, int32 sampInt, int16 scanTime, int32 scanInt)
{						
	AnalogParamBlock apb, *apbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		apbPtr = &apb;
		apb.slot = slot;
		apb.cnt = cnt;
		apb.timebase = sampTime;
		apb.samp_int = sampInt;
		apb.value = scanTime;
		apb.blkCount = scanInt;
		apb.buffer = (LD_Ptr)buffer;
		
		LDSysError = Control(ref_table[slot], SCANIntStartID, (Ptr)&apbPtr);
		ProcError(LDSysError, err, apb.error);
	}
	return err;
} /* SCAN_IntStart */