/*
 * ....................WAVEFORM GENERATION ROUTINES................................
 */

#include "NI_DAQ_MACStructs.h"
#include "OpenBoard.h"

/*
 * Here are routines for asynchronous (WF_***) and synchronous (WF_Grp_***) waveform generation.
 * Note: WF_Check and WF_Load are the same for both modes
 */
 
 
/* ...................Asynchronous routines........................ */

/*
 * WF_Setup -- saves the update interval and timebase in DAC_chan structure
 * for subsequent waveform generation functions.
 */
 
NBErrType WF_Setup (int16 slot, int16 channel, int32 update_interval, int16 timebase)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		wfpb.update_interval = update_interval;
		wfpb.timebase = timebase;
		
		LDSysError = Control(ref_table[slot], WFSetupID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Setup */
/*
 * WF_Load -- configures the DMA controller and timer\counter chip for 
 *			waveform generation.
 */
 
NBErrType WF_Load (int16 slot, int16 channel, int16 *buffer, int32 cnt, int16 cont)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		wfpb.buffer = (LD_Ptr) buffer;
		wfpb.cnt = cnt;
		wfpb.cont = cont;		
		LDSysError = Control(ref_table[slot], WFLoadID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Load */

/*
 * WF_DBLoad -- configures the DMA controller and timer\counter chip for 
 *			waveform generation. (Also for Double buffered waveform generation)
 */
 
NBErrType WF_DBLoad (int16 slot, int16 channel, int16 *buffer, int32 cnt, int16 cont, int32 timeout)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		wfpb.buffer = (LD_Ptr) buffer;
		wfpb.cnt = cnt;
		wfpb.cont = cont;
		wfpb.cycles = timeout;
		
		LDSysError = Control(ref_table[slot], WFLoadID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_DBLoad */


/*
 * WF_Start -- turns on the counter\timer chip output to initiate waveform generation.
 */
 
NBErrType WF_Start (int16 slot, int16 channel)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		
		LDSysError = Control(ref_table[slot], WFStartID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Start */


/*
 * WF_Check -- checks to see if one cycle of the output waveform has been generated.
 */
 
NBErrType WF_Check (int16 slot, int16 channel, int16 *status)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		
		LDSysError = Control(ref_table[slot], WFCheckID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
		
		*status = wfpb.timebase;
	}
	return err;
} /* WF_Check */


/*
 * WF_Stop -- turns off the counter\timer chip output to temporarily halt 
 *			   waveform generation.
 */
 
NBErrType WF_Stop (int16 slot, int16 channel)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		
		LDSysError = Control(ref_table[slot], WFStopID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Stop */


/*
 * WF_Reset -- turns off the counter\timer chip and releases the DMA channel 
 *			   allocated.
 */
 
NBErrType WF_Reset (int16 slot, int16 channel)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		
		LDSysError = Control(ref_table[slot], WFResetID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Reset */


/*
 * WF_Offset --   Offsets each value in buffer for waveform generation to 0 - 4095.
 *				  Before a waveform buffer is sent to WF_Load, this function should
 *				  be called to offset the buffer values if needed.  WF_Offset simply
 *				  adds 2048 to each buffer value if the board is an NB-MIO-16 and 
 *				  the analog output is configured for bipolar.  No conversion is
 *				  performed if the board in 'slot' contains an NB-A0-6 or the board
 *				  in 'slot' is configured for unipolar analog output.
 *
 */
NBErrType WF_Offset(int16 slot, int16 channel, int32 cnt, int16 *buffer)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = channel;
		wfpb.cnt = cnt;
		wfpb.buffer = (LD_Ptr)buffer;
		
		LDSysError = Control(ref_table[slot], WFOffsetID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Offset */



/* ................Synchronous update routines...................... */

/*
 * WF_Grp_Setup -- allocates DMA channels for the group and stores update_interval
 *				   and timebase in DAC_chan configuration.
 */
 
NBErrType WF_Grp_Setup (int16 slot, int16 num_chans, int16 *chan_vector, int32 update_interval, int16 timebase)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		wfpb.channel = num_chans;
		wfpb.buffer = (LD_Ptr) chan_vector;
		wfpb.update_interval = update_interval;
		wfpb.timebase = timebase;
		
		LDSysError = Control(ref_table[slot], WFGrpSetupID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Grp_Setup */


/*
 * WF_Grp_Start -- starts the synchronous mode waveform generation.
 */
 
NBErrType WF_Grp_Start (int16 slot)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;

	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		
		LDSysError = Control(ref_table[slot], WFGrpStartID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Grp_Start */


/*
 * WF_Grp_Stop -- stops the synchronous mode waveform generation. Sync waveform
 *				  generation may be restarted with WF_Grp_Start.
 */
 
NBErrType WF_Grp_Stop (int16 slot)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		
		LDSysError = Control(ref_table[slot], WFGrpStopID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Grp_Stop */


/*
 * WF_Grp_Reset -- stops synchronous mode waveform generation and releases 
 *				   DMA channels.
 */
 
NBErrType WF_Grp_Reset (int16 slot)
{
	WFParamBlock wfpb, *wfpbPtr;
	register NBErrType err;
	
	/* conditionally open slot */
	if ((err = OpenBoard(slot)) == noErr) {
		wfpbPtr = &wfpb;
		wfpb.slot = slot;
		
		LDSysError = Control(ref_table[slot], WFGrpResetID, (Ptr)&wfpbPtr);
		ProcError(LDSysError, err, wfpb.error);
	}
	return err;
} /* WF_Grp_Reset */

/* ................Buffered waveform routines...................... */

/*
 *	BWF_BlkLoad
 */
NBErrType BWF_BlkLoad (int16 slot, int16 group, int16 channel, int32 buf_number, int16 copy_mode, int16 *data, int32 count, int32 start_pt, 
						int32 block_reps, int32 timeout, int32 *pts_copied, int32 *last_pt, int32 *block_reps_done)
/*int16	slot, group, channel, copy_mode, *data;
long	buf_number, count, start_pt, block_reps, timeout, 
		*pts_copied, *last_pt, *block_reps_done;*/
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
	 	Bwf.slot = slot;
	 	Bwf.group = group;
	 	Bwf.channel = channel;
	 	Bwf.buf_num = buf_number;
	 	Bwf.mode = copy_mode;
	 	Bwf.buffer = (Ptr) data;
	 	Bwf.cnt = count;
		Bwf.start_pt = start_pt;
		Bwf.cycles = block_reps;
		Bwf.update_interval = timeout;
		
		LDSysError = Control(ref_table[slot], BWF_BlkLoadID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
		/* return values */
		*pts_copied = Bwf.cnt;
		*last_pt = Bwf.block_size;
		*block_reps_done = Bwf.cycles;
	}
	return err;
} /* BWF_BlkLoad */

/*
 *	BWF_BufLoad
 */
NBErrType BWF_BufLoad (int16 slot, int16 group, int16 channel, int32 buf_number, int32 cycles, int16 *data, int32 count, int32 update_int, int16 timebase, int16 trig_mode, int32 block_size, int32 regen_mode)
										/*int16	slot, group, channel, *data, timebase, trig_mode;
																long	buf_number, cycles, count, update_int, block_size, regen_mode;*/
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
	 	Bwf.slot = slot;
	 	Bwf.group = group;
	 	Bwf.channel = channel;
	 	Bwf.buf_num = buf_number;
	 	Bwf.cycles = cycles;
	 	Bwf.buffer = (Ptr) data;
	 	Bwf.cnt = count;
	 	Bwf.update_interval = update_int;
	 	Bwf.timebase = timebase;
	 	Bwf.cont = trig_mode;
	 	Bwf.block_size = block_size;
	 	Bwf.regen_mode = regen_mode;
	
		LDSysError = Control(ref_table[slot], BWF_BufLoadID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
	}
	return err;
} /* BWF_BufLoad */

/*
 *	BWF_Check
 */
NBErrType BWF_Check (int16 slot, int16 group, int16 channel, int16 check_mode, int16 *status, int32 *buf_number, int32 *cycles_output, int32 *points_output, int32 *update_int, int16 *timebase)
/*int16		slot, group, channel, check_mode, *status, *timebase;
																		long	*buf_number, *cycles_output, *points_output, *update_int;*/
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
	 	Bwf.slot = slot;
	 	Bwf.group = group;
		Bwf.channel = channel;
	 	Bwf.mode = check_mode;
		LDSysError = Control(ref_table[slot], BWF_CheckID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
		/*return values*/
		*status = Bwf.cont;
	 	*buf_number = Bwf.buf_num;
	 	*cycles_output = Bwf.cycles;
	 	*points_output = Bwf.cnt;
	 	*update_int = Bwf.update_interval;
	 	*timebase = Bwf.timebase;
	}
	return err;
} /* BWF_Check */

/*
 *	BWF_Clear
 */
NBErrType BWF_Clear (int16 slot, int16 group, int16 channel)
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
 		Bwf.slot = slot;
 		Bwf.group = group;
 		Bwf.channel = channel;
	
		LDSysError = Control(ref_table[slot], BWF_ClearID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
	}
	return err;
} /* BWF_Clear */

/*
 *	BWF_Rate
 */
NBErrType BWF_Rate (int16 slot, int16 group, int16 channel, int32 update_int, int16 timebase,int16 mode)
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
 		Bwf.slot = slot;
	 	Bwf.group = group;
	 	Bwf.channel = channel;
	 	Bwf.update_interval = update_int;
	 	Bwf.timebase = timebase;
	 	Bwf.mode = mode;
	
		LDSysError = Control(ref_table[slot], BWF_RateID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
	}
	return err;
} /* BWF_Rate */

/*
 *	BWF_Start
 */
NBErrType BWF_Start (int16 slot, int16 group, int16 channel, int32 *buf_numbers, int16 start_mode)
/*int16		slot, group, channel, start_mode;
long	*buf_numbers;*/
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
 		Bwf.slot = slot;
	 	Bwf.group = group;
	 	Bwf.channel = channel;
	 	Bwf.buffer = (Ptr) buf_numbers;
	 	Bwf.mode = start_mode;
	 	
		LDSysError = Control(ref_table[slot], BWF_StartID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
	}
	return err;
} /* BWF_Start */

/*
 *	BWF_Stop
 */
NBErrType BWF_Stop (int16 slot, int16 group, int16 channel, int16 stop_mode)
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
 		Bwf.slot = slot;
	 	Bwf.group = group;
	 	Bwf.channel = channel;
	 	Bwf.mode = stop_mode;
	 	
		LDSysError = Control(ref_table[slot], BWF_StopID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
	}
	return err;
} /* BWF_stop */

/*
 *	BWF_Resume
 */
NBErrType BWF_Resume (int16 slot, int16 group, int16 channel)
{	
	WFParamBlock Bwf, *BwfPtr;	/* allocate Multiple buffered waveform generation parameter block */
	register NBErrType err;
	
	if ((err = OpenBoard(slot)) == noErr) {
		BwfPtr = &Bwf;
 		Bwf.slot = slot;
 		Bwf.group = group;
 		Bwf.channel = channel;
	
		LDSysError = Control(ref_table[slot], BWF_ResumeID, (Ptr)&BwfPtr);
		ProcError(LDSysError, err, Bwf.error);
		
	}
	return err;
} /* BWF_Resume */