/* hist.c:	C - MEX file to speed up the M-file hist.m
 * function [n,x] = hist(y,x)
 * 
 * This C-MEX-file computes histograms in less than 3 seconds on a
 * 12 MHz PC-AT (length of input vector: 8000), independent of the
 * number of bins chosen. The hist.m M-file takes one minute or even more,
 * depending on the number of bins used.
 * 
 * Caution: The original hist.m only computed histograms, they were displayed
 * using plot or other funtions. With PC-Matlab 3.50, a call to hist.m
 * without return parameters given plots the histogram. This MEX-file 
 * is functionally equivalent to the M-file, except that it does NO PLOTTING:
 * You have to call it using
 * 	[n,x] = hist(...)
 * and use some plotting function, e.g. bar(x,n), to display the result.
 * 
 * This file was compiled with Microsoft C 5.0 on a PC-AT using the
 * following batch file:
 * 
 * CMEX350.BAT:
 * > rem  compile MATLAB MEX files with Microsoft C 5.0 and 5.1.
 * > rem !!!!!! enter the location of YOUR matlab\mex directory below: !!!!
 * > @echo off
 * > set MLM=d:\matlab\mex
 * > 
 * > set oldCL=%CL%
 * > set CL=
 * > CL /AL /Aw /c /Gs /FPi87 /W3 %1.c
 * > IF ERRORLEVEL 1 GOTO DONE
 * > LINK %MLM%\CMEX+%1+%MLM%\CMISCDAT,%1.MEX /M /NOE;
 * > %MLM%\PROT_MEX %1
 * > DEL %1.OBJ
 * > DEL %1.MAP
 * > :DONE
 * > set CL=%oldCL%
 * > set oldCL=
 * > 
 *	
 * 
 * Author: Richard L. Reng  Jan-1991
 * 
 * with:   Lehrstuhl fuer Nachrichtentechnik, University of Erlangen,
 *         Cauerstrasse 7, D-8520 Erlangen, FRG
 *         phone:  (+49) 9131-85-7116    FAX:   (+49) 9131-30 38 40
 *         E-mail: reng@nt.e-technik.uni-erlangen.de
 */


#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "cmex.h"  /* cmex.h is included with the Matlab shipment */

/* Input Arguments */

#define	Y_IN	prhs[0]
#define X_IN	prhs[1]

/* Output Arguments */

#define	N_OUT	plhs[0]
#define	X_OUT	plhs[1]


/* some preprocessor definitions */

#define	min(A, B)	((A) < (B) ? (A) : (B))
#define mrows(A)	((A)->m)
#define ncols(A)	((A)->n)
#define realdata(A)	((A)->pr)

void user_fcn(int, Matrix **, int, Matrix **);

void user_fcn(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix **plhs, **prhs;
{
	double	*y_in, *x_in, *n_out, *x_out;
	int	nbin;		/* number of bins */
	int	imax = nbin -1, index;
	int	y_length;	/* length of input data */
	int	x_is_scalar = 0;
	int	k;		/* loop counter */
	double	ymin, ymax;	/* y range */
	double	delta;		/* (ymax - ymin) / nbin */
	double	t0, t1;		/* used in index transformation */

	/* Check for proper number of arguments */
	if ( nrhs < 1 || nrhs > 2 )
		mex_error("HIST.MEX: wrong number of input arguments.");
	if ( nlhs < 1 || nlhs > 2 )
		mex_error("HIST.MEX: wrong number of output arguments.");

	y_in  = realdata(Y_IN);
	y_length  = mrows(Y_IN) * ncols(Y_IN);

	if (nrhs == 1) {
		nbin = 10;
		x_is_scalar = 1;
	}
	else if (mrows(X_IN) == 1 && ncols(X_IN) == 1) { /* x is scalar */
		x_in  = realdata(X_IN);
		nbin = (int) x_in[0];
		x_is_scalar = 1;
	}
	else if (min(mrows(X_IN), ncols(X_IN)) != 1)
		mex_error("HIST.MEX: x must be scalar or vector.");
	else {
		x_in  = realdata(X_IN);
		nbin = mrows(X_IN) * ncols(X_IN);
	}

	imax = nbin - 1;

	N_OUT =	create_matrix(1, nbin, REAL);
	n_out = realdata(N_OUT);

	if (nlhs == 2) {
		X_OUT = create_matrix(1, nbin, REAL);
		x_out = realdata(X_OUT);
	}

	if (x_is_scalar) {
		/* get minimum and maximum element of y */
	   double *yy = y_in + ( k = y_length-1 );
	   ymin = ymax = *yy;
	   while (k--) {
		(*--yy < ymin) ? ymin = *yy :( (*yy > ymax) ? ymax = *yy :0.0);
	   }

	   delta = (ymax - ymin) / nbin;
	   if (nlhs == 2)
	      for (k=0; k < nbin; k++) {
		x_out[k] = ymin + ((double)k + 0.5) * delta;
	      }
	}
	else {
	   delta = x_in[1] - x_in[0];
	   ymin  = x_in[0] - 0.5 * delta;
	   ymax  = x_in[imax] + 0.5 * delta;

	   if (nlhs == 2)
	      for (k=0; k < nbin; k++)
		x_out[k] = x_in[k];
	}

	/* compute constants for the linear transformation
	 * of data values to histogram counter indices */
	t1 = 1.0 / delta;
	t0 = - t1 * ymin;

	/* bad trick (10% speed improvement):
	 * The histogram counters are incremented as integers and later
	 * converted to double (requires sizeof(int) <= sizeof(double)).
	 * In case of problems, remove the (int*) casts below and
	 * delete the 'convert results' FOR loop. Also, initialize with
	 * 0.0 instead of 0.
	 */
	for (k=0; k < nbin; k++)
		*((int*)n_out + k) = 0;		/* 0.0 if (double*) */

		/* after all that administration,
		 * we shouldn't forget to compute the histogram ... */
	for (k=0; k < y_length; k++) {
		index = (int) floor(t0 + t1*y_in[k]);
		(index>imax) ? ++*((int*)n_out+imax) : ((index<0) ? 
			++*((int*)n_out) : ++*((int*)n_out + index) );
	}

		/* convert result from int to double */
	for (k = imax; k >= 0; k--)
		n_out[k] = (double) *((int*)n_out + k);;

	/* done */
}
/***** end of hist.c *****/

